#!/bin/bash
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted
# material is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Milan Zazrivec

# Description: SELinux file contexts scan, reports files with wrong labels
# Concept:
#     1) First we'll find out if there's SELinux enabled
#     2) Create the list of partitions to be scanned
#     3) Find out the SELINUXTYPE we're using (targeted/strict)
#     4) Perform the actual scan using setfiles, parse its output and rearrange
#        it into some printable form
#     5) Report the results

echo "Running SELinux file context test."
echo
echo "Purpose of this test is to find mislabeled files on all currently"
echo "mounted filesystems. Test will perform file scan and check wether all"
echo "file security labels match the current SELinux policy."
echo ""
echo

echo -n "Checking if we are running with SELinux enabled: "
# does it make sense to run the test?
/usr/sbin/selinuxenabled
if [ ! $? -eq 0 ]; then
    echo "FAIL"
    exit 1;
else
    echo "OK"
fi

# idea taken from fixfiles utility (policycoreutils pkg.) written by Dan Walsh

SF_OUT=/var/tmp/setfiles.out
# PATTERN is the line-pattern produced by setfiles
PATTERN='^.*setfiles\: \+relabeling \(.\+\) from \(.\+\) to \(.\+\)$'
# SUBST is the output pattern
SUBST='FILE: "\1" WAS: "\2" EXPECTED: "\3"'

# what filesystems do we want to scan? -> FILESYSTEMS
FS_RW=`mount | grep -v "context=" | egrep -v '\((|.*,)bind(,.*|)\)' |\
       awk '/(ext[23]| xfs | jfs ).*\(rw/{print $3}';`
FS_RO=`mount | grep -v "context=" | egrep -v '\((|.*,)bind(,.*|)\)' |\
       awk '/(ext[23]| xfs | jfs ).*\(ro/{print $3}';`
FILESYSTEMS="$FS_RW $FS_RO"

# what SELinux mode are we currently running?
SELINUXMODE="permissive"
if [ `cat /selinux/enforce` -eq 1 ]; then
    SELINUXMODE="enforcing"
fi

# what SELINUXTYPE do we use?
SELINUXTYPE="targeted"
if [ -e /etc/selinux/config ]; then
    . /etc/selinux/config
    FC=/etc/selinux/${SELINUXTYPE}/contexts/files/file_contexts
else
    FC=/etc/security/selinux/file_contexts
fi

echo "We are running SELinux in $SELINUXMODE mode, $SELINUXTYPE policy."
if [ -f $FC ]; then
    echo "Found specification file: $FC"
else
    echo "Specification file does not exist"
    exit 1;
fi

echo
echo "Performing scan..."

rm -f $SF_OUT
# actual scan, print only error mesages|grep 'lines containing relabeling...'|
# |do the sed substitution (nice looking output) >> write it to /var/tmp/...
/usr/sbin/setfiles -v -q -n $FC $FILESYSTEMS | grep 'relabeling' | \
    sed s/"$PATTERN"/"$SUBST"/ >> $SF_OUT

# how many files are there to relabel? -> CNT
CNT=`cat $SF_OUT | wc -l`

# print results
if [ $CNT -gt 0 ]; then
    echo "Found $CNT files with wrong labels: "
    cat $SF_OUT
fi

rm -f $SF_OUT

# set results
if [ $CNT -eq 0 ]; then
    exit 0
else
    exit 1
fi
